@@ -1,19 +1,18 @@
 import re
-from operator import itemgetter
-from collections import defaultdict, namedtuple, OrderedDict
+import typing
+import pathlib
+import operator
 import warnings
+import collections
 
 
 import pybtex.database
-from clldutils.path import readlines
 from clldutils import jsonlib
 from clldutils.misc import lazyproperty
 from clldutils.apilib import API
 from clldutils.source import Source
-try:
-    import cldfcatalog
-except ImportError:  # pragma: no cover
-    cldfcatalog = None
+from clldutils.markup import iter_markdown_tables
+import cldfcatalog
 
 from pyconcepticon.util import read_dicts, lowercase, to_dict, UnicodeWriter, split, BIB_PATTERN
 from pyconcepticon.glosses import concept_map, concept_map2
@@ -24,6 +23,8 @@ from pyconcepticon.models import (  # noqa: F401
     REF_PATTERN, compare_conceptlists, MD_SUFFIX,
 )
 
+Editor = collections.namedtuple('Editor', ['name', 'start', 'end'])
+
 
 class Concepticon(API):
     """
@@ -36,54 +37,39 @@ class Concepticon(API):
         'url': 'https://concepticon.clld.org',
         'title': 'Concepticon',
         'description': 'A Resource for the Linking of Concept Lists',
-        'publisher.name': 'Max Planck Institute for the Science of Human History',
-        'publisher.place': 'Jena',
-        'publisher.url': 'https://www.shh.mpg.de',
-        'publisher.contact': 'concepticon@shh.mpg.de',
+        'publisher.name': 'Max Planck Institute for Evolutionary Anthropology',
+        'publisher.place': 'Leipzig',
+        'publisher.url': 'https://www.eva.mpg.de',
+        'publisher.contact': 'concepticon@eva.mpg.de',
     }
 
-    def __init__(self, repos=None):
+    def __init__(self, repos: typing.Optional[typing.Union[str, pathlib.Path]] = None):
         """
         :param repos: Path to a clone or source dump of concepticon-data.
         """
-        if (repos is None) and cldfcatalog:
-            repos = cldfcatalog.Config.from_file().get_clone('concepticon')
+        repos = repos or cldfcatalog.Config.from_file().get_clone('concepticon')
         API.__init__(self, repos)
         self._to_mapping = {}
 
-    def data_path(self, *comps):
+    def data_path(self, *comps: str) -> pathlib.Path:
         """
         Create a path relative to the `concepticondata` directory within the source repos.
         """
         return self.path('concepticondata', *comps)
 
     @lazyproperty
-    def editors(self):
+    def editors(self) -> typing.List[Editor]:
         res = []
-        Editor = namedtuple('Editor', ['name', 'start', 'end'])
-        in_editors, in_table = False, False
-        for line in readlines(self.path('CONTRIBUTORS.md'), strip=True):
-            if in_editors and line.startswith('#'):
-                in_editors, in_table = False, False
-                continue
-
-            if line.endswith('# Editors'):
-                in_editors = True
-                continue
-
-            if in_editors and line.startswith('--- '):
-                in_table = True
-                continue
-
-            if in_table and '|' in line:
-                period, _, name = line.partition('|')
-                period = period.strip().partition('-')
-                res.append(
-                    Editor(name.strip(), period[0].strip(), period[2].strip() or None))
+        header, rows = next(
+            iter_markdown_tables(self.path('CONTRIBUTORS.md').read_text(encoding='utf8')))
+        for (period, name) in rows:
+            start, to_, end = period.strip().partition('-')
+            start, end = start.strip(), end.strip()
+            res.append(Editor(name.strip(), start, start if not to_ else end or None))
         return res
 
     @lazyproperty
-    def vocabularies(self):
+    def vocabularies(self) -> typing.Dict[str, dict]:
         """
         Provide access to a `dict` of controlled vocabularies.
         """
@@ -95,19 +81,20 @@ class Concepticon(API):
         return res
 
     @property
-    def bibfile(self):
+    def bibfile(self) -> pathlib.Path:
         return self.data_path('references', 'references.bib')
 
     @lazyproperty
-    def sources(self):
+    def sources(self) -> dict:
         return jsonlib.load(self.data_path('sources', 'cdstar.json'))
 
     @lazyproperty
     def retirements(self):
-        return jsonlib.load(self.data_path('retired.json'), object_pairs_hook=OrderedDict)
+        return jsonlib.load(
+            self.data_path('retired.json'), object_pairs_hook=collections.OrderedDict)
 
     def add_retirement(self, type_, repl):
-        obj = OrderedDict()
+        obj = collections.OrderedDict()
         for k in ['id', 'comment', 'replacement']:
             obj[k] = repl[k]
             assert obj[k]
@@ -117,19 +104,16 @@ class Concepticon(API):
         jsonlib.dump(self.retirements, self.data_path('retired.json'), indent=2)
 
     @lazyproperty
-    def bibliography(self):
+    def bibliography(self) -> typing.Dict[str, Source]:
         """
         :returns: `dict` mapping BibTeX IDs to `Reference` instances.
         """
-        refs = []
-        with self.bibfile.open(encoding='utf8') as fp:
-            for key, entry in pybtex.database.parse_string(
-                    fp.read(), bib_format='bibtex').entries.items():
-                refs.append(Source.from_entry(key, entry))
-        return to_dict(refs)
+        return to_dict(
+            Source.from_entry(key, entry) for key, entry in pybtex.database.parse_string(
+                self.bibfile.read_text(encoding='utf8'), bib_format='bibtex').entries.items())
 
     @lazyproperty
-    def conceptsets(self):
+    def conceptsets(self) -> typing.Dict[str, Conceptset]:
         """
         :returns: `dict` mapping ConceptSet IDs to `Conceptset` instances.
         """
@@ -169,7 +153,7 @@ class Concepticon(API):
             meta=md,
             values=to_dict(
                 read_dicts(values_path, schema=md['tableSchema']),
-                key=itemgetter('CONCEPTICON_ID')))
+                key=operator.itemgetter('CONCEPTICON_ID')))
 
     @lazyproperty
     def relations(self):
@@ -187,7 +171,7 @@ class Concepticon(API):
 
     @lazyproperty
     def frequencies(self):
-        D = defaultdict(int)
+        D = collections.defaultdict(int)
         for cl in self.conceptlists.values():
             for concept in cl.concepts.values():
                 if concept.concepticon_id:
@@ -276,12 +260,23 @@ class Concepticon(API):
         if out is None:
             print(writer.read().decode('utf-8'))
 
-    def lookup(self, entries, full_search=False, similarity_level=5, language='en'):
+    def lookup(
+            self,
+            entries,
+            full_search=False,
+            similarity_level=5,
+            language='en',
+            mincsid=None,
+            to=None,
+    ):
         """
         :returns: `generator` of tuples (searchterm, concepticon_id, concepticon_gloss, \
         similarity).
         """
-        to = self._get_map_for_language(language, None)
+        if to is None:
+            to = self._get_map_for_language(language, None)
+            if mincsid:
+                to = [t for t in to if int(t[0]) >= mincsid]
         tox = [i[1] for i in to]
         cfunc = concept_map2 if full_search else concept_map
         cmap = cfunc(
@@ -321,12 +316,23 @@ class Concepticon(API):
                 print(_msg('error', msg, name, line))
             return not bool(errors)
 
-        if errors:
-            return exit()
+        if errors:  # pragma: no cover
+            return exit()  # Exit early in case of structural errors.
 
-        REF_WITHOUT_LABEL_PATTERN = re.compile(r'[^\]]\(:(ref|bib):[A-Za-z0-9\-]+\)')
+        REF_WITHOUT_LABEL_PATTERN = re.compile(r'[^]]\(:(ref|bib):[A-Za-z0-9\-]+\)')
         REF_WITHOUT_LINK_PATTERN = re.compile('[^(]:(ref|bib):[A-Za-z0-9-]+')
 
+        # Make sure all language-specific mappings are well specified
+        iso_langs = [
+            lang.iso2 for lang in self.vocabularies['COLUMN_TYPES'].values()
+            if isinstance(lang, Languoid) and lang.iso2]
+        if len(iso_langs) != len(set(iso_langs)):
+            error(
+                'Duplicate ISO codes: {}'.format(collections.Counter(iso_langs).most_common(1)),
+                'concepticon.json')
+        assert set(p.stem.split('-')[1] for p in self.path('mappings').glob('map-*.tsv'))\
+            .issubset(iso_langs)
+
         # We collect all cite keys used to refer to references.
         all_refs = set()
         refs_in_bib = set(ref for ref in self.bibliography)
@@ -339,11 +345,11 @@ class Concepticon(API):
                 if set(value.keys()) != cnames_schema:  # pragma: no cover
                     error('meta data {0} contains irregular number of columns in line {1}'
                           .format(meta.id, i + 2), 'name')
-                if key not in self.conceptsets:
+                if key not in self.conceptsets:  # pragma: no cover
                     error('meta data {0} references invalid CONCEPTICON_ID {2} in line {1}'
                           .format(meta.id, i + 2, key), 'name')
             for ref in split(meta.meta.get('dc:references') or ''):
-                if ref not in refs_in_bib:
+                if ref not in refs_in_bib:  # pragma: no cover
                     error('cited bibtex record not in bib: {0}'.format(ref), 'name')
                 all_refs.add(ref)
 
@@ -351,7 +357,7 @@ class Concepticon(API):
         # concept lists.
         for i, cl in enumerate(self.conceptlists.values()):
             if clids and cl.id not in clids:
-                continue
+                continue  # pragma: no cover
             fl = ('conceptlists.tsv', i + 2)
             for ref in re.findall(BIB_PATTERN, cl.note) + cl.refs:
                 if ref not in refs_in_bib:
@@ -362,11 +368,11 @@ class Concepticon(API):
             for m in REF_WITHOUT_LABEL_PATTERN.finditer(cl.note):
                 error('link without label: {0}'.format(m.string[m.start():m.end()]), *fl)
 
-            for m in REF_WITHOUT_LINK_PATTERN.finditer(cl.note):
+            for m in REF_WITHOUT_LINK_PATTERN.finditer(cl.note):  # pragma: no cover
                 error('reference not in link: {0}'.format(m.string[m.start():m.end()]), *fl)
 
             for m in REF_PATTERN.finditer(cl.note):
-                if m.group('id') not in self.conceptlists:
+                if m.group('id') not in self.conceptlists:  # pragma: no cover
                     error('invalid conceptlist ref: {0}'.format(m.group('id')), *fl)
 
             # make also sure that all sources are accompanied by a PDF, but only write a
@@ -378,7 +384,7 @@ class Concepticon(API):
 
         if not clids:
             # Only report unused references if we check all concept lists!
-            for ref in refs_in_bib - all_refs:
+            for ref in refs_in_bib - all_refs:  # pragma: no cover
                 error('unused bibtex record: {0}'.format(ref), 'references.bib')
 
         ref_cols = {
@@ -399,7 +405,7 @@ class Concepticon(API):
 
         for fname in self.data_path('conceptlists').glob('*.tsv'):
             if clids and fname.stem not in clids:
-                continue
+                continue  # pragma: no cover
             if fname.stem not in self.conceptlists:  # pragma: no cover
                 error(
                     'conceptlist missing in conceptlists.tsv: {0}'.format(fname.name), '')
@@ -407,6 +413,8 @@ class Concepticon(API):
         broken_cls = []
 
         for cl in self.conceptlists.values():
+            if clids and cl.id not in clids:
+                continue  # pragma: no cover
             #
             # Check consistency between the csvw metadata and the column names in the list.
             #
@@ -414,12 +422,12 @@ class Concepticon(API):
             cols_in_md = []
             for col in cl.metadata.tableSchema.columns:
                 cnames = []  # all names or aliases csvw will recognize for this column
-                if col.name in cols_in_md:
+                if col.name in cols_in_md:  # pragma: no cover
                     error('Duplicate name ot title in table schema: {0}'.format(col.name), cl.id)
                 cnames.append(col.name)
                 if col.titles:
                     c = col.titles.getfirst()
-                    if c in cols_in_md:
+                    if c in cols_in_md:  # pragma: no cover
                         error('Duplicate name ot title in table schema: {0}'.format(c), cl.id)
                     cnames.append(c)
                 cols_in_md.extend(cnames)
@@ -445,9 +453,9 @@ class Concepticon(API):
 
                     if concept.concepticon_id:
                         cs = self.conceptsets.get(concept.concepticon_id)
-                        if not cs:
+                        if not cs:  # pragma: no cover
                             error('invalid conceptset ID %s' % concept.concepticon_id, cl.id)
-                        elif cs.gloss != concept.concepticon_gloss:
+                        elif cs.gloss != concept.concepticon_gloss:  # pragma: no cover
                             error(
                                 'wrong conceptset GLOSS for ID {0}: {1} -> {2}'.format(
                                     cs.id, concept.concepticon_gloss, cs.gloss),
@@ -469,13 +477,13 @@ class Concepticon(API):
                             # check that there are not leading and trailing spaces
                             # (while computationally expensive, this helps catch really
                             # hard to find typos)
-                            if val != val.strip():
+                            if val != val.strip():  # pragma: no cover
                                 error("leading or trailing spaces in value for %s: '%s'" %
                                       (attr, val), cl.id, i + 2)
 
                             if val not in values:  # pragma: no cover
                                 error('invalid value for %s: %s' % (attr, val), cl.id, i + 2)
-            except TypeError as e:
+            except TypeError as e:  # pragma: no cover
                 broken_cls.append(cl.id)
                 error(str(e), cl.id)
 
@@ -503,7 +511,7 @@ class Concepticon(API):
 
         for cl in self.conceptlists.values():
             if cl.id in broken_cls:
-                continue
+                continue  # pragma: no cover
             for concept in cl.concepts.values():
                 if concept.concepticon_id in deprecated:  # pragma: no cover
                     error('deprecated concept set {0} linked for {1}'.format(
